// Emacs style mode select   -*- C++ -*-
//-----------------------------------------------------------------------------
//
// $Id:$
//
// Copyright (C) 1993-1996 by id Software, Inc.
//
// This source is available for distribution and/or modification
// only under the terms of the DOOM Source Code License as
// published by id Software. All rights reserved.
//
// The source is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// FITNESS FOR A PARTICULAR PURPOSE. See the DOOM Source Code License
// for more details.
//
// $Log:$
//
// DESCRIPTION:
//
//-----------------------------------------------------------------------------

/***************************************************************************
                       DOOMWORLD
***************************************************************************/

static const char
rcsid[] = "$Id: m_bbox.c,v 1.1 1997/02/03 22:45:10 b1 Exp $";

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <conio.h>

#include <netinet/in.h>
#include <errno.h>
#include <unistd.h>
#include <dpmi.h>
#include <go32.h>
#include <sys/nearptr.h>
#include <libnet.h>

#include "i_system.h"
#include "d_event.h"
#include "d_net.h"
#include "m_argv.h"

#include "doomstat.h"

#ifdef __GNUG__
#pragma implementation "i_net.h"
#endif
#include "i_net.h"

#define SIZE           1000
#define DOOM_MAGIC     0x6a
#define MAXPLAYERS    8

void	NetSend (void);
boolean NetListen (void);


//
// NETWORKING
//

void	(*netget) (void);
void	(*netsend) (void);

NET_CHANNEL *listenchan, *chan[MAXPLAYERS];
char buffer[SIZE], address[1024], target[1024];
int netdriver = 1;			// Winsock
int intendedplayers;

//
// I_CloseChannels
//
void I_CloseChannels()
{
	// close all channels
	int i;

	net_closechannel (listenchan);
	for (i=0; i<MAXPLAYERS; i++)
	{
		net_closechannel (chan[i]);
	}
}

//
// I_PacketReady
//
int I_PacketReady()
{
	int   i;

	for (i=0; i<MAXPLAYERS; i++)
	{
		if (net_query(chan[i])) return i;
	}
	return -1;
}

//
// I_InitWinsock
//
void I_InitWinsock ()
{
	char    addr[1024];
	int     i,x;

	if ((!net_initdrivers (netdriver)) && netdriver) {
		I_Error ("Can't initialize Winsock\n");
	}

	// open all channels
	for (i=0; i<MAXPLAYERS; i++)
	{
		if (!(chan[i] = net_openchannel (netdriver))) {
			I_CloseChannels ();
			I_Error ("Unable to open channel.\n");
		}
	}

	printf ("Winsock initialized.\n");

	if (doomcom->consoleplayer)
	{
		// Need something to connect to
//		printf ("Enter target address: ");
//		gets (addr);
		strcpy (addr,target);
		printf ("Connecting to %s...\n", addr);

		// Just use chan[0] since we are client
		net_assigntarget (chan[0], addr);

		// Send a 1 byte packet to the server
		sprintf (buffer,"%c",DOOM_MAGIC);
		net_send (chan[0], buffer, 1);

		while (!net_query (chan[0]))
			if (kbhit())
			{
				if (getch() == 27)
				{
					I_Error ("Internet connection aborted.\n");
				}
			}
		printf ("Connection accepted ");
		x = net_receive (chan[0], buffer, SIZE, address);
		if (x == -1)
		{
			I_Error ("I_InitWinsock: Receive error\n");
		}
		printf ("(%s)\n",address);
		doomcom->consoleplayer = buffer[0];

		printf ("id: %d\n",doomcom->consoleplayer);
		// Reassign to new port
		net_assigntarget (chan[0], address);

		printf ("Waiting for game: ");
		while (!net_query (chan[0]))
			if (kbhit())
			{
				if (getch() == 27)
				{
					I_Error ("Internet connection aborted.\n");
				}
			}
		x = net_receive (chan[0], buffer, SIZE, address);
		if (x == -1)
		{
			I_Error ("I_InitWinsock: Receive error\n");
		}
		doomcom->numnodes = doomcom->numplayers = buffer[0];
		printf("%d\n",doomcom->numplayers);
	}

	else {
		doomcom->numplayers = doomcom->numnodes = 1;
		listenchan = net_openinputchannel (netdriver);
		if (!listenchan) {
			I_Error ("I_InitWinsock: Couldn't open input channel.\n");
		}
		while (doomcom->numplayers < intendedplayers)
		{
			// We need to wait for a connection
			printf ("Waiting for connection %d: ",doomcom->numplayers);
			while (!net_query(listenchan)) if (kbhit()) break;

			if (kbhit())
			{
				I_CloseChannels ();
				I_Error ("I_InitWinsock: Internet connection aborted.\n");
			}

			if (net_query(listenchan))		// This should always be true
			{
				int n;

				x = net_receive (listenchan, buffer, SIZE, address);

				if (x == -1)
				{
					I_CloseChannels ();
					I_Error ("Receive error.\n");
				}

				n = doomcom->numplayers-1;

				if (net_assigntarget (chan[n], address)) {
					I_CloseChannels ();
					I_Error ("I_InitWinsock: net_assigntarget failed.\n");

				}
				printf ("%s connected (channel %d)\n",address,n);

				// Assign them a player number
				doomcom->numplayers++;
				buffer[0] = n+1;
//                              printf ("(player %d, channel %d)\n",n+1,n);

				net_send (chan[n], buffer, 1);
			}
		}
		// Tell them numplayers
		buffer[0] = doomcom->numplayers;

		for (i=0; i<doomcom->numplayers-1; i++)
		{
			printf("sent start packet to channel %d\n",i);
			net_send (chan[i], buffer, 1);
		}
	}
	doomcom->numnodes = doomcom->numplayers;
	if (M_CheckParm ("-ps"))
	{
		printf ("netgame: %d, consoleplayer: %d, numnodes: %d, numplayers: %d\n",
			netgame,doomcom->consoleplayer,doomcom->numnodes,doomcom->numplayers);
		getchar ();
	}
}


//
// PacketSend
//
void PacketSend (void)
{
	if (doomcom->consoleplayer)
		net_send (chan[0], &doomcom->data, doomcom->datalength);
	else
		net_send (chan[doomcom->remotenode-1], &doomcom->data, doomcom->datalength);
}


//
// PacketGet
//
void PacketGet (void)
{
      char    localbuffer[SIZE];

      int     i,x;

      doomcom->remotenode = -1;

      i = I_PacketReady();
      if (i < 0)		     		// No packet ready
      {
	    return;
      }

      x = net_receive (chan[i], localbuffer, SIZE, NULL);
      if (x == -1)
      {
	    I_Error ("PacketGet: Read error.\n");
      }
      if (!x)
      {
	    I_Error ("PacketGet: net_query reports data, but net_receive reports none.\n");
      }
      memcpy (&doomcom->data, localbuffer, x);
      doomcom->remotenode = 1;
      doomcom->datalength = x;
}

//
// I_InitNetwork
//
void I_InitNetwork (void)
{
    int			i,j;

    // set up for network

    // parse network game options,
    //  -net <consoleplayer> <host> <host> ...
    i = M_CheckParm ("-net");
    if (!i)
      {
      // single player game
      doomcom = malloc (sizeof (*doomcom) );
      memset (doomcom, 0, sizeof(*doomcom) );

      netgame = false;
      doomcom->id = DOOMCOM_ID;
      doomcom->numplayers = doomcom->numnodes = 1;
      doomcom->deathmatch = false;
      doomcom->consoleplayer = 0;
      doomcom->extratics=0;
      doomcom->ticdup=1;
      return;
      }

    // multi-player game
    doomcom = malloc (sizeof (*doomcom) );
    memset (doomcom, 0, sizeof(*doomcom) );

    if ( (strchr(myargv[i+1],'.')) == NULL)
    {
	// (player 0, green)
	doomcom->consoleplayer = 0;
	intendedplayers = atoi (myargv[i+1]);
    }
    else {
	// Need something to connect to
	doomcom->consoleplayer = 1;		// Temporary
	strcpy (target,myargv[i+1]);
    }

    doomcom->id = DOOMCOM_ID;

//    doomcom->numnodes = doomcom->numplayers = 2;
//    doomcom->consoleplayer = atoi(myargv[i+1])-1;

    //
    //	Game stats
    //
    doomcom->deathmatch = true;

    doomcom->ticdup=1;
    if (M_CheckParm ("-extratic"))
      doomcom-> extratics = 1;
    else
      doomcom-> extratics = 0;

    j = M_CheckParm ("-dup");
    if (j && j< myargc-1)
      {
      doomcom->ticdup = myargv[j+1][0]-'0';
      if (doomcom->ticdup < 1)
	doomcom->ticdup = 1;
      if (doomcom->ticdup > 9)
	doomcom->ticdup = 9;
      }
    else
      doomcom-> ticdup = 1;

    netsend = PacketSend;
    netget = PacketGet;
    netgame = true;
    I_InitWinsock ();
}


void I_NetCmd (void)
{
    if (doomcom->command == CMD_SEND)
    {
	netsend ();
    }
    else if (doomcom->command == CMD_GET)
    {
	netget ();
    }
    else
	I_Error ("Bad net cmd: %i\n",doomcom->command);
}

