/*----------------------------------------------------------------
 * config.c - configuration file routines for libnet
 *----------------------------------------------------------------
 *  libnet is (c) Copyright Chad Catlett and George Foot 1997-1998
 *
 *  Please look in `docs' for details, documentation and
 *  distribution conditions.
 */

#ifdef DJGPP
#include <crt0.h>
#endif
#include <ctype.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/stat.h>

#include "libnet.h"
#include "drivers.h"
#include "config.h"

static char default_config_filename[] =
#ifdef DJGPP
"libnet.cfg";
#else
".libnetrc";
#endif


/* net_loadconfig:
 *	Loads a config file for the network drivers.  Pass a filename, 
 *	directory or NULL.  If you pass only a filename it is assumed to be
 *	in the current directory.  If you pass a directory, the filename is
 *	taken to be default_config_filename above.  If you pass NULL, this
 *	loads deafult_config_filename from the program's directory.  Returns 
 *	zero on success.
 */
int net_loadconfig (char *filename) {
	int retval, type;
	char *chp0, *chp1;
	FILE *fp;
	NET_DRIVER *drv;
	struct stat st;

	if (filename == NULL)
	  filename = default_config_filename;
	else {
	  if (stat (filename, &st)) return 1;                           /* file not found? */
	  if (S_ISDIR (st.st_mode)) {					/* directory */
		strcat (filename, "/");
		strcat (filename, default_config_filename);
	  }
	}

	fp = fopen (filename, "rt");
	if (!fp) return 2; 									/* file opening error */

	type = 1;
	while ((drv = __net_get_driver (type))) {
		drv->load_config (fp);
		type <<= 1;
	}

	fclose (fp);
	return 0;
}

/* My version of stricmp */
static int __gf__stricmp__gf__ (char *a, char *b) {
  int i;
  for (i = 0; a[i]; i++) {
	if (a[i] < b[i]) return -1;
	if (a[i] > b[i]) return 1;
  }
  return b[i] ? -1 : 0;
}


/* __libnet_internal__seek_section:
 *	This function is used internally; it searches the given open text 
 *	file `fp' for a line containing only the string `section' inside
 *	square brackets.  Returns zero on success.
 */
int __libnet_internal__seek_section (FILE *fp, char *section) {
	char buffer [1024];
	char target [1024];

	strcpy (target, "[");
	strcat (target, section);
	strcat (target, "]\n");

	rewind (fp);
	do {
		fgets (buffer, 1024, fp);
		if (strlen (buffer) == 1023)
			while ((strlen (buffer) == 1023) && fgets (buffer, 1024, fp));
		else {
			if (!__gf__stricmp__gf__ (buffer, target)) return 0;
		}
	} while (!feof (fp));

	return 1;
}

/* __libnet_internal__get_setting:
 *	This internal function is used to read the first "option = value"
 *	pair from `fp'.  It strips leading and trailing whitespace from
 *	both `*option' and `*value', and stops looking if it finds a line
 *	starting with `['.  Returns zero if it found a pair.
 */
int __libnet_internal__get_setting (FILE *fp, char **option, char **value) {
	static char buffer [1024];
	char *chp;

	if ((!option) || (!value) || (!fp)) return 1;

	do {
		fgets (buffer, 1024, fp);
		if (strlen (buffer) == 1023) {
			while ((strlen (buffer) == 1023) && fgets (buffer, 1024, fp));
			buffer [0] = 0;
		}
	} while ((!feof (fp)) && ((buffer[0] == '#') || ((buffer[0] != '[') && (!strchr (buffer, '=')))));

	if ((buffer[0] != '#') && (chp = strchr (buffer, '='))) {
		*option = buffer;
		*value = chp + 1;
		*chp = 0;

		while (isspace (**value)) (*value)++;
		chp = strchr (*value, 0) - 1;
		while ((chp >= *value) && isspace (*chp)) *chp-- = 0;

		while (isspace (**option)) (*option)++;
		chp = strchr (*option, 0) - 1;
		while ((chp >= *option) && isspace (*chp)) *chp-- = 0;

		return 0;
	}

	return 1;
}
