/*----------------------------------------------------------------
 * streams.c - stream routines for libnet
 *----------------------------------------------------------------
 *  libnet is (c) Copyright Chad Catlett and George Foot 1997-1998
 *
 *  Please look in `docs' for details, documentation and
 *  distribution conditions.
 */


/* net_createstream:
 *  Creates a new stream.  Returns NULL on error.
 */
NET_STREAM *net_createstream() {
	NET_STREAM *retval = (NET_STREAM *) malloc (sizeof (NET_STREAM));

	retval->sourcelist = NULL;

	return retval;
}


/* net_stream_addsource:
 *  Adds an input source to a stream.  Returns zero on success.
 */
int net_stream_addsource (NET_STREAM *stream, int networktype, ...) {
	NET_STREAMSOURCELIST *p = (NET_STREAMSOURCELIST *) malloc (sizeof (NET_STREAMSOURCELIST));

	p->type = networktype;
	p->driver = get_driver (p->type);
	p->driver->initsource (p, (&networktype)+1);

	if (stream->sourcelist) {
		p->next = stream->sourcelist->next;
		stream->sourcelist->next = p;
	} else {
		p->next = p;
		stream->sourcelist = p;
	}
}


/* net_stream_getpacket:
 *  Gets a packet from a stream.  Returns number of packets which
 *  were queued.  If no packets were queued, the function returns
 *  zero and clears the NET_PACKET passed.
 */
int net_stream_getpacket (NET_STREAM *stream, NET_PACKET *packet) {
}


/* net_closestream:
 *  Closes a stream.
 */
int net_closestream(NET_STREAM *stream) {
	NET_STREAMSOURCELIST *p, *np;

	if (stream->sourcelist) {
		p = stream->sourcelist->next;
		stream->sourcelist->next = NULL;
		while (p) {
			np = p->next;
			p->driver->destroysource (p);
			p = np;
		}
}



