/*----------------------------------------------------------------
 * chat/server.c - chat server demo for libnet
 *----------------------------------------------------------------
 *  libnet is (c) Copyright Chad Catlett and George Foot 1997-1998
 *
 *  Please look in `docs' for details, documentation and
 *  distribution conditions.
 */

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libnet.h>


#define SIZE 256

NET_CHANNEL *chan;
char buffer[SIZE];
int netdriver = 0;


void list_drivers (NET_DRIVERNAME *drvs, char *title) {
	printf("\nDriver list: %s\n",title);
	while (drvs->name) {
		printf("\t%d\t%s\n",drvs->num,drvs->name);
		drvs++;
	}
	printf ("\n");
}


int in_list (NET_DRIVERNAME *drvs, int x) {
	while (drvs->name) {
		if (x==drvs->num) return 1;
		drvs++;
	}
	return 0;
}


void get_driver() {
	char buffer[20];
	int avail,choice;
	NET_DRIVERNAME *drivers;

	printf ("Detecting available drivers...\n");
	avail = net_detectdrivers (NET_DRIVER_ALL);
	printf ("Getting detected driver names...\n");
	drivers = net_getdrivernames (avail);

	do {
		list_drivers (drivers, "Available drivers");

		printf ("Please choose a driver from the above list.\n");
		fgets (buffer,10,stdin);
		choice = atoi (buffer);
	} while (!in_list (drivers, choice));
	free (drivers);

	netdriver = choice;
}


void init() {
  if ((!net_initdrivers (netdriver)) && netdriver) {
    printf ("Error initialising driver.\n");
    exit (1);
  }

  printf ("Identify yourself: ");
  fgets (name, SIZE, stdin);

  do {
    printf ("Shall I be the server? (Y/N) ");
    fgets (buffer, SIZE, stdin);
    buffer[0] = tolower (buffer[0]);
  } while ((buffer[0] != 'y') && (buffer[0] != 'n'));

  if (buffer[0] == 'y') {
    chan = net_openinputchannel (netdriver);
    printf ("Waiting for incoming connection...\n");
    do {
      x = net_receive (chan, buffer, SIZE-1, address);
      if (x == -1) x++;
      buffer[x] = 0;
    } while (!strcmp (buffer, "hello"));
    net_assigntarget (chan, address);
    net_send (chan, "hi", 2, address);
    do {
      x = net_receive (chan, buffer, SIZE-1, address);
      if (x == -1) x++;
      buffer[x] = 0;
    } while (!buffer[0]);
    strcpy (opponent_name, buffer);
    net_send (chan, name, strlen (name));
    
	
}


char *get_message() {
	chan_list_t *chanlist = channellist;
	int x=0;
	char *buf2;

	while (chanlist && !x) if (!(x=net_query (chanlist->chan))) chanlist = chanlist->next;
	if (x) {
		strcpy (buffer, chanlist->nick);
		strcat (buffer, ": ");
		buf2 = strchr (buffer, 0);
		x = net_receive (chanlist->chan, buf2, SIZE-1-(buf2-buffer), NULL);
		buf2[x] = 0;

		if (buf2[0] == '/') {

			if (!stricmp (buf2+1,"quit")) {
				chan_list_t *p1 = NULL;

				if (chanlist != channellist)
					for (p1 = channellist; p1 && (p1->next != chanlist); p1 = p1->next);

				sprintf (buffer, "--- %s left", chanlist->nick);
				dispatch_message (buffer);

				strcpy (buffer, "*** go away");
				net_send (chanlist->chan, buffer, strlen (buffer));

				if (p1) p1->next = chanlist->next; else channellist = chanlist->next;
				net_closechannel (chanlist->chan);
				free (chanlist->nick);
				free (chanlist);
			}

			return NULL;
		} else return buffer;
	} else return NULL;
}


void check_for_new_connections() {
	char address[32], *newnick;
	int x = net_receive (listenchan, buffer, SIZE-1, address);

	if (x && (buffer[0] == CHAT_MAGIC)) {
		chan_list_t *chanlist = (chan_list_t *) malloc (sizeof (chan_list_t));
		buffer[x] = 0;
		if (!(chanlist->chan = net_openchannel (netdriver))) {
			sprintf (buffer, "!!! failed to open channel for %s",address);
			dispatch_message (buffer);
			free (chanlist);
			return;
		}
		if (net_assigntarget (chanlist->chan, address)) {
			sprintf (buffer, "!!! couldn't assign target `%s' to channel",address);
			dispatch_message (buffer);
			net_closechannel (chanlist->chan);
			free (chanlist);
			return;
		}
		newnick = strdup (buffer+1);
		sprintf (buffer, "+++ %s joined from %s", newnick, address);
		dispatch_message (buffer);
		strcpy (buffer, "OK");
		net_send (chanlist->chan, buffer, strlen (buffer));
		chanlist->nick = newnick;
		chanlist->next = channellist;
		channellist = chanlist;
	}
}


void do_server() {
	char *message;
	do {
		message = get_message();
		if (message) dispatch_message (message);
		check_for_new_connections();
	} while (!kbhit());
	getch();
	sprintf (buffer, "*** server shutting down");
	dispatch_message (buffer);
}


int main(void) {
	printf ("libnet chat server version %d.%d\n", VER_MAJOR, VER_MINOR);
	printf ("built at " __TIME__ " on " __DATE__ "\n");
	printf ("\n");

	net_init();
	net_loadconfig(NULL);
	get_driver();
	init();

	printf ("*** server awaiting connections\n");
	do_server();

	return 0;
}

