/*----------------------------------------------------------------
 * libnet.h - interface to libnet library
 *----------------------------------------------------------------
 *  libnet is (c) Copyright Chad Catlett and George Foot 1997-1998
 *
 *  Please look in `docs' for details, documentation and
 *  distribution conditions.
 */


#ifndef libnet_included_file_libnet_h
#define libnet_included_file_libnet_h


#include <stdio.h>


typedef struct NET_DRIVER NET_DRIVER;
typedef struct NET_CHANNEL NET_CHANNEL;
typedef struct NET_DRIVERNAME NET_DRIVERNAME;
typedef struct NET_PACKET NET_PACKET;
typedef struct NET_STREAM NET_STREAM;
typedef struct NET_STREAMSOURCELIST NET_STREAMSOURCELIST;


struct NET_CHANNEL {
	int type;                      /* network type */
	struct NET_DRIVER *driver;     /* network driver */
	char *target_addr;             /* target address */
	char *return_addr;             /* return address */
	void *data;                    /* data block for driver's use */
};

struct NET_DRIVER {
	char *name;                                                        /* driver name */
	char *desc;                                                        /* description string */

	int (*detect)();                                                   /* auto-detect function (0 = absent) */
	int (*init)();                                                     /* initialise (0 = okay) */
	int (*exit)();                                                     /* undo the above initialisation */

	int (*init_channel) (NET_CHANNEL *chan);                           /* perform low-level initialisation on a channel */
	int (*init_inputchannel) (NET_CHANNEL *chan);                      /* initialise a channel to receive connections */
	int (*destroy_channel) (NET_CHANNEL *chan);                        /* undo the above initialisation */

	int (*update_target) (NET_CHANNEL *chan);                          /* update private data for change of target address */
	int (*send) (NET_CHANNEL *chan, void *buf, int size);              /* send data */
	int (*recv) (NET_CHANNEL *chan, void *buf, int max, char *from);   /* receive data */
	int (*query) (NET_CHANNEL *chan);                                  /* query for incoming data */

	void (*load_config) (FILE *fp);                                    /* load configuration data */
};

struct NET_PACKET {
	int nettype;                   /* network type */
	int source;                    /* source of packet */
	int size;                      /* size of data in bytes */
	void *data;                    /* pointer to data */
};

struct NET_STREAMSOURCELIST {
	NET_STREAMSOURCELIST *next;    /* pointer to next */
	int type;                      /* network type */
	struct NET_DRIVER *driver;     /* network driver */
	void *data;                    /* data block for driver's use */
};

struct NET_STREAM {
	NET_STREAMSOURCELIST *sourcelist;   /* (circular) list of sources this reads from */
};

struct NET_DRIVERNAME {
	int num;                    /* reference number */
	char *name;                 /* driver name */
};


#define NET_MAX_ADDRESS_LENGTH	128

#define NET_DETECT_YES          2
#define NET_DETECT_MAYBE        1
#define NET_DETECT_NO           0


#define NET_DRIVER_NONET      0        /* No networking */
#define NET_DRIVER_WSOCK      1        /* Winsock */
#define NET_DRIVER_SOCKETS    2        /* Berkeley sockets (on Unices) */

#define NET_DRIVER_ALL        3        /* all drivers */
#define NET_DRIVER_ALL_USEFUL 3        /* all useful drivers */


int net_init ();
NET_DRIVERNAME *net_getdrivernames (int which);
int net_detectdrivers (int which);
int net_initdrivers (int which);
int net_shutdown ();
NET_CHANNEL *net_openchannel (int type);
int net_assigntarget (NET_CHANNEL *channel, char *target);
int net_closechannel (NET_CHANNEL *channel);
int net_send (NET_CHANNEL *channel, void *buffer, int size);
int net_receive (NET_CHANNEL *channel, void *buffer, int maxsize, char *from);
int net_query (NET_CHANNEL *channel);
char *net_getlocaladdress (NET_CHANNEL *channel);
NET_CHANNEL * net_openinputchannel (int type);
int net_loadconfig (char *filename);


#endif

